import java.util.Enumeration;
import java.util.Vector;
import javax.swing.text.Segment;
import org.gjt.sp.jedit.syntax.KeywordMap;

public class ParserRuleSet
{
	public ParserRuleSet(int sz)
	{
		rules = new Vector(sz);
	}

	public ParserRuleSet()
	{
		this(DEFAULT_RULE_VECTOR_SIZE);
	}

	public void addRule(ParserRule r)
	{
		rules.addElement(r);
	}

	public void removeRule(ParserRule r)
	{
		rules.removeElement(r);
	}

	public Enumeration getRules()
	{
		return rules.elements();
	}

	public ParserRule[] getRuleArray()
	{
		ParserRule[] ruleArray = new ParserRule[rules.size()];
		rules.copyInto(ruleArray);
		return ruleArray;
	}

	public int getRuleCount()
	{
		return rules.size();
	}

	public int getTerminateChar()
	{
		return terminateChar;
	}

	public void setTerminateChar(int atChar)
	{
		terminateChar = (atChar >= 0) ? atChar : -1;
	}

	public boolean getIgnoreCase()
	{
		return ignoreCase;
	}

	public void setIgnoreCase(boolean b)
	{
		ignoreCase = b;
	}

	public KeywordMap getKeywords()
	{
		return keywords;
	}

	public void setKeywords(KeywordMap km)
	{
		keywords = km;
	}

	public ParserRule getEscapeRule()
	{
		return escapeRule;
	}

	public Segment getEscapePattern()
	{
		if (escapePattern == null && escapeRule != null)
		{
			escapePattern = new Segment(escapeRule.searchChars, 0,
				escapeRule.sequenceLengths[0]);
		}
		return escapePattern;
	}

	public void setEscape(String esc)
	{
		if (esc == null)
		{
			escapeRule = null;
		}
		else
		{
			escapeRule = ParserRuleFactory.createEscapeRule(esc,
				GenericTokenMarker.NULL);
		}
		escapePattern = null;
	}

	public int getDefault()
	{
		return defaultID;
	}

	public void setDefault(int def)
	{
		defaultID = def;
	}

	private static final int DEFAULT_RULE_VECTOR_SIZE = 16;
	private KeywordMap keywords;
	private Vector rules;
	private ParserRule escapeRule;
	private Segment escapePattern;
	private int terminateChar = -1;
	private boolean ignoreCase = true;
	private int defaultID;
}
