/*
 * XModeOptionPane.java - Option pane for XMode
 * Copyright (C) 1999 mike dillon
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.MiscUtilities;
import org.gjt.sp.jedit.AbstractOptionPane;
import org.gjt.sp.jedit.OptionPane;

import org.gjt.sp.util.Log;

public class XModeOptionPane extends AbstractOptionPane
	implements ActionListener
{
	// public members
	public XModeOptionPane()
	{
		super("xmode");
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		JPanel northPanel = new JPanel(new BorderLayout(5,0));
		northPanel.setOpaque(false);
		JPanel labelPanel = new JPanel(new GridLayout(0,1));
		labelPanel.setOpaque(false);
		JPanel fieldPanel = new JPanel(new GridLayout(0,1));
		fieldPanel.setOpaque(false);
		northPanel.add(labelPanel, BorderLayout.WEST);
		northPanel.add(fieldPanel, BorderLayout.CENTER);
		add(northPanel, BorderLayout.NORTH);

		JLabel label = new JLabel(jEdit.getProperty(
			"options.xmode.grammar-directory"));
		labelPanel.add(label);

		JPanel directoryPanel = new JPanel(new BorderLayout(5,0));
		grammarDirectory = new JTextField();
		label.setLabelFor(grammarDirectory);
		grammarDirectory.setText(jEdit.getProperty(
			"xmode.grammar-directory"));
		directoryPanel.add(grammarDirectory, BorderLayout.CENTER);

		pickDirectory = new JButton(new ImageIcon(jEdit.class
			.getResource("/org/gjt/sp/jedit/toolbar/" +
			jEdit.getProperty("open-file.icon"))));
		pickDirectory.setMargin(new Insets(0,0,0,0));
		pickDirectory.setToolTipText(jEdit.getProperty(
			"options.xmode.choose-directory"));
		pickDirectory.addActionListener(this);
		directoryPanel.add(pickDirectory, BorderLayout.EAST);

		fieldPanel.add(directoryPanel);
	}

	// begin OptionPane implementation
	public void save()
	{
		jEdit.setProperty("xmode.grammar-directory",
			grammarDirectory.getText());
	}
	// end OptionPane implementation

	// begin ActionListener implementation
	public void actionPerformed(ActionEvent evt)
	{
		JFileChooser chooser = new JFileChooser();
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int retVal = chooser.showDialog(this, jEdit.getProperty(
			"options.xmode.choose-directory"));
		if(retVal == JFileChooser.APPROVE_OPTION)
		{
			File file = chooser.getSelectedFile();
			if(file != null)
			{
				try
				{
					String dirName = file.getCanonicalPath();
					grammarDirectory.setText(dirName);
				}
				catch(IOException e)
				{
					Log.log(Log.ERROR, this, e);
				}
			}
		}
	}
	// end ActionListener implementation

	// private members
	private JButton pickDirectory;
	private JTextField grammarDirectory;
}
