/*
 * XModePlugin.java - Extensible Mode Plugin
 * Copyright (C) 1999 mike dillon
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;

import org.gjt.sp.jedit.DefaultErrorSource;
import org.gjt.sp.jedit.EBPlugin;
import org.gjt.sp.jedit.EditBus;
import org.gjt.sp.jedit.ErrorSource;
import org.gjt.sp.jedit.jEdit;

import org.gjt.sp.jedit.gui.OptionsDialog;


public class XModePlugin extends EBPlugin
{
	public static final String name = "XMode";

	// public members
	// begin Plugin implementation
	public void start()
	{
		errorSource = new DefaultErrorSource(this.name);
		EditBus.addToNamedList(ErrorSource.ERROR_SOURCES_LIST,
			errorSource);
		EditBus.addToBus(errorSource);

		String xmodeGrammars = jEdit.getProperty(
			"xmode.grammar-directory");
		if (xmodeGrammars == null)
		{
			xmodeGrammars = System.getProperty("user.home") +
				File.separator + ".jedit" +
				File.separator + "modes" +
				File.separator + "xml";
			jEdit.setProperty("xmode.grammar-directory",
				xmodeGrammars);
		}
		loadGrammars(xmodeGrammars);
	}

	public void stop()
	{
	}

	public void createOptionPanes(OptionsDialog optionsDialog)
	{
		optionsDialog.addOptionPane(new XModeOptionPane());
	}

	// package-private members
	static void addError(String file, int lineIndex, String error)
	{
		errorSource.addError(ErrorSource.ERROR, file, lineIndex, 0, 0,
			error);
	}

	static void clearErrors()
	{
		errorSource.clear();
	}

	// private members
	private void loadGrammars(String directory)
	{
		File file = new File(directory);
		if(!(file.exists() && file.isDirectory())) return;

		String[] grammars = file.list();

		if(grammars == null) return;

		String grammar, fileName;

		FileReader reader;

		for(int i = 0; i < grammars.length; i++)
		{
			grammar = grammars[i];
			if(!grammar.toLowerCase().endsWith(".xml"))
				continue;

			fileName = directory + File.separator + grammar;

			reader = null;
			try
			{
				reader = new FileReader(new File(fileName));
			}
			catch (FileNotFoundException e)
			{
				// should not happen
			}

			if (reader != null)
			{
				xmode newMode = XModeFactory.createXMode(
					reader, fileName);
				if (newMode != null)
					jEdit.addMode(newMode);
			}
		}
	}

	private static DefaultErrorSource errorSource;
}
